///////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2002 Jason Baldridge and Gann Bierner
// Modified for supertagging where indicated
// Changes (C) 2004 Richard Moot (Richard.Moot@labri.fr)
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//////////////////////////////////////////////////////////////////////////////

import opennlp.common.morph.*;
import opennlp.common.util.*;
import opennlp.maxent.*;

import opennlp.tools.util.BeamSearchContextGenerator;


import java.io.*;
import java.util.*;

/**
 * A context generator for the supertagger.
 * Modified from the POS context generator by Gann Bierner     20040203 RM
 *
 * @author      Gann Bierner
 * @version     $Revision: 1.1.1.1 $, $Date: 2005/02/02 18:08:44 $
 */

public abstract class AbstractContextGenerator implements BeamSearchContextGenerator {

    protected String[] context;
    protected ArrayList e;

    public String[] getContext(Object o) {
	Object[] data = (Object[]) o;
	return getContext(((Integer) data[3]).intValue(), (Object[]) data[0], (Object[]) data[1], (String[]) data[2], null);
  }

    public String[] getContext(int index, Object[] sequence, String[] priorDecisions, Object[] additionalContext) {
	Pair p = splitSequence(sequence);
	return getContext(index, (Object[]) p.a, (Object[]) p.b, priorDecisions, additionalContext);
    }


    public String[] getContext(int index, Object[] words, Object[] tags, String[] stags, Object[] additionalContext) {
	e = auxGetContext(index, words, tags, stags, additionalContext);
	String[] context = new String[e.size()];
        e.toArray(context);
	return context;
    }

    abstract protected ArrayList auxGetContext(int index, Object[] words, Object[] tags, String[] stags, Object[] additionalContext);

    public Pair splitSequence(Object[] pairs) {
	int end = pairs.length;

	String[] lw = new String[end];
	String[] lp = new String[end];

	for(int i=0; i<end; i++) {
	    String p = pairs[i].toString();

	    int s = p.indexOf("/");
	    lw[i] = p.substring(0,s);
	    lp[i] = p.substring(s+1);
	}
	return new Pair(lw, lp);
    }

    public static Pair[] mergeSequences(String[] first, String[] second) {
	int end = first.length;

	Pair[] pairs = new Pair[end];

	for(int i=0; i<end; i++) {
	    pairs[i] = new Pair(first[i],second[i]);
	}
	return pairs;
    }
}
