//////////////////////////////////////////////////////////////////////////////
// A little class for evaluating the result of a maximum entropy model.
// (C) 2004 Richard Moot (Richard.Moot@labri.fr)
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//////////////////////////////////////////////////////////////////////////////


import java.io.*;
import java.util.*;

import org.jdom.*;

import opennlp.common.preprocess.*;
import opennlp.common.xml.*;
import opennlp.common.util.*;

import opennlp.maxent.*;
import opennlp.maxent.io.*;

class EvaluationME {

    public static String removeResults(String ts) {


	StringBuffer sb = new StringBuffer();
	StringTokenizer st = new StringTokenizer(ts);
	String s = null;
	int i;

	while(st.hasMoreTokens()) {
	    s = st.nextToken();
	    i = s.lastIndexOf("/");
	    sb.append(s.substring(0,i)+" ");
	}

	return sb.toString().trim();
    }

    public static void main(String[] args) {
	try {
	    int solutions, beamsize;
	    if (args.length >= 3) {
		if (args.length >= 4) {
		    solutions = Integer.parseInt(args[3]);
		} else {
		    solutions = 1;
		}
		System.out.println("Solutions: "+solutions);
		if (args.length >= 5) {
		    beamsize = Integer.parseInt(args[4]);
		} else {
		    beamsize = 3*solutions;
		}
		System.out.println("Beam size: "+beamsize);
		System.out.print("Reading model...");
                TriSuperTaggerME st = new TriSuperTaggerME(beamsize, new SuffixSensitiveGISModelReader(new File(args[2])).getModel(), new TriContextGenerator());
		System.out.println("done!");
		BufferedReader inf = new BufferedReader(new FileReader(args[0]));
		PrintWriter outf = new PrintWriter(new FileWriter(args[1]));

		String line = null;

		while ((line=inf.readLine()) != null) {
		    line = removeResults(line);
		    outf.println(st.supertag(solutions, line.replace('_','/')));
		    System.out.print(".");
		}

		System.out.println();
		outf.close();
		inf.close();

	    } else {
		System.out.println("Usage: java EvaluationME infile outfile model");
	    }
	} catch (IOException e) {
	    System.err.println(e);
	    System.exit(1);
	}
    }
}