import java.io.*;
import java.util.*;

import org.jdom.*;

import opennlp.common.preprocess.*;
import opennlp.common.xml.*;
import opennlp.common.util.*;

import opennlp.maxent.*;
import opennlp.maxent.io.*;

/**
 * A supertagger that uses maximum entropy. Straightforward adaptation
 * of the part-of-speech tagger by Gann Bierner.
 */

public class Supertag {

    public static void main(String[] args) {

	try {
	    if (args.length >= 3) {
		int solutions, beamsize;

		if (args.length >= 4) {
		    solutions = Integer.parseInt(args[3]);
		} else {
		    solutions = 1;
		System.out.println("Solutions: "+solutions);
		}
		if (args.length >= 5) {
		    beamsize = Integer.parseInt(args[4]);
		} else {
		    beamsize = 3*solutions;
		}
		System.out.println("Beam size: "+beamsize);
		System.out.print("Reading model...");
                TriSuperTaggerME st = new TriSuperTaggerME(beamsize, new SuffixSensitiveGISModelReader(new File(args[2])).getModel(), new TriContextGenerator());
		System.out.println("done!");
		BufferedReader inf = new BufferedReader(new FileReader(args[0]));
		PrintWriter outf = new PrintWriter(new FileWriter(args[1]));

		String line;

		while ((line=inf.readLine()) != null) {
		    outf.println(st.supertag(solutions, line.replace('_','/')));
		    System.out.print(".");
		}

		System.out.println();
		outf.close();
		inf.close();

		System.exit(0);

	    } else {
		System.out.println("Usage: java Supertag infile outfile model [solutions] [beamsize]");
	    }
	} catch (IOException e) {
	    System.err.println(e);
	    System.exit(1);
	}
    }
}
